<?php
namespace Sale\Handlers\PaySystem;
use Bitrix\Sale;
use Bitrix\Catalog;
use Bitrix\Catalog\Mysql\CCatalogVat;
use Bitrix\Main\Application;
use Bitrix\Main\Web\Uri;
use Bitrix\Main\Error;
use Bitrix\Main\Request;
use Bitrix\Main\Type\DateTime;
use Bitrix\Main\Localization\Loc;
use Bitrix\Sale\PaySystem;
use Bitrix\Sale\Payment;
use Bitrix\Sale\PriceMaths;
class PayKeeperHandler extends PaySystem\ServiceHandler
{
    private $fiscal_cart = array();
    private $payment_form_url = "";
	/**
	 * @param Payment $payment
	 * @param Request|null $request
	 * @return PaySystem\ServiceResult
	 */
	public function initiatePay(Payment $payment, Request $request = null)
	{
		$busValues = $this->getParamsBusValue($payment);
        $form_url = $busValues["PAYKEEPER_FORM_URL"];
        $this->setPaymentFormUrl($form_url);
        $secret = $busValues["PAYKEEPER_SECRET"];
        $orderid = ($this->getPaymentFormType() == "create") ? $payment->getOrderId() : $payment->getOrderId() . "|paykeeper_payment_system";
        $order = Sale\Order::load($orderid);
        $propertyCollection = $order->getPropertyCollection();
        $total_sum = number_format(floatval($order->getPrice()), 2, ".", ""); 
        $clientid = is_null($propertyCollection->getPayerName()) ? "" : $propertyCollection->getPayerName()->getValue();
        $client_email = is_null($propertyCollection->getUserEmail()) ? "" : $propertyCollection->getUserEmail()->getValue();
        $client_phone = is_null($propertyCollection->getPhone()) ? "" : $propertyCollection->getPhone()->getValue();
        $service_name = "paykeeper_payment_system";
        $basket = $order->getBasket();
        foreach ($basket as $item) {
            $taxes = array("tax" => "none", "tax_sum" => 0);
            $name = $item->getField("NAME");
            $quantity = $item->getField("QUANTITY");
            $price = $item->getField("PRICE");
            $sum = $price*$quantity;
            $vat_rate = (int) ($item->getField("VAT_RATE")*100);
            $taxes = $this->setTaxes($sum, (float) $vat_rate);
            $this->updateFiscalCart($name, $price, $quantity, $sum, $taxes["tax"], $taxes["tax_sum"]);
        }
        $dbSaleDelivery = \Bitrix\Sale\Delivery\Services\Manager::getById($order->getField("DELIVERY_ID"));
        $delivery_name = $dbSaleDelivery["NAME"];
        $delivery_price = $order->getDeliveryPrice();
        //$delivery_tax_obj = CCatalogVat::GetById($dbSaleDelivery["VAT_ID"]);
        //$delivery_tax_rate = (int)$delivery_tax_obj->Fetch()["RATE"];
        $delivery_tax_rate = 18;
        $delivery_taxes = ((int) $dbSaleDelivery["VAT_ID"] == 0) ?
                            array("tax" => "none", "tax_sum" => 0) :
                            $this->setTaxes($delivery_price, $delivery_tax_rate);
        if ($delivery_price > 0) $this->updateFiscalCart(
                                    $delivery_name,
                                    number_format($delivery_price, 2, ".", ""),
                                    1,
                                    number_format($delivery_price, 2, ".", ""),
                                    $delivery_taxes["tax"],
                                    $delivery_taxes["tax_sum"]);
        //Encode fiscal cart to utf-8 for json_encode
        $cart_encoded = array();
        foreach ($this->fiscal_cart as $product) {
            $product_ar = array();
            foreach ($product as $key => $value) {
                $enc = mb_detect_encoding($value, 'ASCII, UTF-8, windows-1251', false);
                $product_ar[$key] = ($enc == "UTF-8") ? $value : iconv($enc, "UTF-8", $value);
            }
            $cart_encoded[] = $product_ar;
        }
        $this->fiscal_cart = json_encode($cart_encoded);
		$params = array(
            "form_url" => $form_url,
            "sum" => $total_sum,
            "orderid" => $orderid,
            "clientid" => $clientid,
            "client_email" => $client_email,
            "client_phone" => $client_phone,
            "service_name" => $service_name,
            "phone" => $phone,
            "cart" => $this->fiscal_cart
        );
        $request = Application::getInstance()->getContext()->getRequest();
        $uriString = $request->getRequestUri();
        $uri = new Uri($uriString);
        $detail_page_active = strpos($uri->GetLocator(), "/personal/order/detail/");
        //payment form language
        switch (LANGUAGE_ID) {
            case "en":
                $pay_button = "Pay online";
                if ($detail_page_active !== false) {
                    $message = "Press pay button for redirect to bank payment gateway now.";
                    $params["detail_page_message"] = "Press pay button on personal account page for redirect to bank payment gateway now.";
                }
                else
                    $message = "You will be redirected to bank payment gateway now.";
                $pf_lang = "en";
                break;
            default:
                $pay_button = "Оплатить";
                if ($detail_page_active !== false) {
                    $message = "Нажмите на кнопку Оплатить для перенаправления на страницу банка.";
                    $params["detail_page_message"] = "Нажмите на кнопку Оплатить в персональном разделе сайта для перенаправления на страницу банка.";
                }
                else
                    $message = "Сейчас Вы будете перенаправлены на страницу банка.";
                $pf_lang = "ru";
                break;
        }
        if (LANG_CHARSET != "utf-8" and $pf_lang == "ru")
            $message = iconv("UTF-8", LANG_CHARSET, $message);
        $params["message"] = $message;
        if ($this->getPaymentFormType() == "create") { //create form
            $params["form_type"] = "create";
            $to_hash = $total_sum    .
                       $clientid     .
                       $orderid      .
                       $service_name .
                       $client_email .
                       $client_phone .
                       $secret;
            $sign = hash ('sha256' , $to_hash);
            $params["sign"] = $sign;
            $params["lang"] = $pf_lang;
            $pay_button = (LANG_CHARSET == "UTF-8") ? $pay_button :
                      iconv("UTF-8", LANG_CHARSET, $pay_button);
            $params["pay_button"] = $pay_button;
            $js_submit_string = ($uri->GetLocator() == "/personal/order/payment/"
                || $detail_page_active !== false) ?
                NULL : 'document.forms["pay_form"].submit();';
            $params["js_submit_string"] = $js_submit_string;
        }
        else { //order form
            $payment_parameters = array(
                "clientid"=>$clientid,
                "orderid"=>$orderid,
                "sum"=>$total_sum,
                "phone"=>$client_phone,
                "client_email"=>$client_email,
                "cart"=>$this->fiscal_cart);
            $query = http_build_query($payment_parameters);
            if( function_exists( "curl_init" )) { //using curl
                $CR = curl_init();
                curl_setopt($CR, CURLOPT_URL, $form_url);
                curl_setopt($CR, CURLOPT_POST, 1);
                curl_setopt($CR, CURLOPT_FAILONERROR, true); 
                curl_setopt($CR, CURLOPT_POSTFIELDS, $query);
                curl_setopt($CR, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($CR, CURLOPT_SSL_VERIFYPEER, 0);
                $result = curl_exec( $CR );
                $error = curl_error( $CR );
                if( !empty( $error )) {
                    $form = "<br/><span class=message>"."INTERNAL ERROR:".$error."</span>";
                    return false;
                }
                else {
                    $form = $result;
                }
                curl_close($CR);
            }
            else { //using file_get_contents
                if (!ini_get('allow_url_fopen')) {
                    $form = "<br/><span class=message>"."INTERNAL ERROR: Option allow_url_fopen is not set in php.ini"."</span>";
                }
                else
                    $form = file_get_contents($form_url, false, $context);
            }
            if ($form  == "") {
                $form = '<h3>Произошла ошибка при инциализации платежа</h3>';
            }
            if ($detail_page_active === false)
                echo $form;
        }
		$this->setExtraParams($params);
        return $this->showTemplate($payment, "template");
	}
	/**
	 * @return array
	 */
	public static function getIndicativeFields()
	{
		return array('id', 'sum', 'clientid', 'orderid', 'key');
	}
    /** 
     * @param Request $request
     * @param $paySystemId
     * @return bool
     */
    static protected function isMyResponseExtended(Request $request, $paySystemId)
    {   
        $orderid = $request->get('orderid');
        $service_name = $request->get('service_name');
        if ($service_name == 'paykeeper_payment_system')
            return TRUE;
        $orderid_split = explode('|', $orderid);
        if (is_array($orderid_split))
            if (count($orderid_split) > 1)
                if ($orderid_split[1] == 'paykeeper_payment_system')
                    return TRUE;
        return FALSE;
    }
	/**
	 * @param Request $request
	 * @return mixed
	 */
	public function getPaymentIdFromRequest(Request $request)
	{
        return $this->getOrderIdFromRequest($request);
	}
	/**
	 * @param Payment $payment
	 * @param Request $request
	 * @return PaySystem\ServiceResult
	 */
	public function processRequest(Payment $payment, Request $request)
	{
        $payment_id = (int) $request->get('id');
        $clientid = $request->get('clientid');
        $sum = (float) $request->get('sum');
        $orderid = $request->get('orderid');
        $sign = $request->get('key');
        if($payment_id == 0)
        {
            echo "No payment specified!";
            die();
        }
		$busValues = $this->getParamsBusValue($payment);
        $secret = $busValues["PAYKEEPER_SECRET"];
        $hash = md5($payment_id.number_format($sum, 2, '.', '').$clientid.$orderid.$secret);
        if ($hash != $sign)
        {
            echo "Hash mismatch";
            die();
        }
        else
        {
            $result = new PaySystem\ServiceResult();
            $fields = array(
                "PS_STATUS" => "Y",
                "PS_STATUS_CODE" => "Success",
                "PS_STATUS_DESCRIPTION" => "Payment accepted",
                "PS_STATUS_MESSAGE" => "Payment id: $payment_id",
                "PS_SUM" => $sum,
                "PS_CURRENCY" => "",
            );
            $result->setOperationType(PaySystem\ServiceResult::MONEY_COMING);
            $result->setPsData($fields);
            echo "OK ".md5($payment_id.$secret);
        }
		return $result;
	}
	/**
	 * @return array
	 */
	public function getCurrencyList()
	{
		return array('RUB');
	}
    private function updateFiscalCart($name = "", $price = 0, $quantity = 0, $sum = 0, $tax = "none", $tax_sum = 0)
    {
        if ($this->getPaymentFormType() == "create") {
            $name = str_replace("\n ", "", $name);
            $name = str_replace("\r ", "", $name);
            $name = str_replace("'", "`", $name);
        }
        $this->fiscal_cart[] = array(
            "name" => $name,
            "price" => number_format($price, 2, ".", ""),
            "quantity" => $quantity,
            "sum" => number_format($sum, 2, ".", ""),
            "tax" => $tax,
            "tax_sum" => number_format($tax_sum, 2, ".", "")
        );
    }
    private function setTaxes($sum, $tax_rate)
    {
        $taxes = array("tax" => "none", "tax_sum" => 0);
        switch($tax_rate) {
            case 10:
                $taxes["tax"] = "vat10";
                $taxes["tax_sum"] = round((float) (($sum/110)*10), 2); 
                break;
            case 18: 
                $taxes["tax"] = "vat18";
                $taxes["tax_sum"] = round((float) (($sum/118)*18), 2); 
                break;
            case 20: 
                $taxes["tax"] = "vat20";
                $taxes["tax_sum"] = round((float) (($sum/120)*20), 2); 
                break;
        }   
        return $taxes;
    }
    private function getPaymentFormType()
    {
        if (strpos($this->getPaymentFormUrl(), "/order/inline") == True)
            return "order";
        else
            return "create";
    }
    private function setPaymentFormUrl($payment_form_url)
    {
        $this->payment_form_url = $payment_form_url;
    }
    private function getPaymentFormUrl()
    {
        return $this->payment_form_url;
    }
    private function getOrderIdFromRequest(Request $request)
    {
        if (strpos($request->get('orderid'), 'paykeeper_payment_system') !== false) {
            $order_data = explode("|", $request->get('orderid'));
            $orderid = $order_data[0];
        }
        else
            $orderid = $request->get('orderid');
        $order = Sale\Order::load($orderid);
        return $order->getPaymentCollection()->current()->getField("ID");
    }
}
